/*
 * twlog:  A basic ham loging program using Motif
 * Copyright (C) 1997->2016 Ted Williams - WA0EIR 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have recieved a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge. MA 02139,
 * USA. See the COPYING file in this directory.
 *
 */

/*
 * TWLOG CALLBACKS FUNCTIONS
 */

#include "twlog.h"

#define MAXBOX 20
#define MAXNOTE 80

int send_eqsl(char abox[][MAXBOX+1]);

extern Pixmap pixmap;        /* pixmap for the application icon */

/*
 * dateCB Callback
 * Puts the date into the date text field and forces 
 * focus to the startPB.
 */
void dateCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Cdata *wids = (Cdata *) cdata;
   time_t current;
   struct tm *timestruct = NULL;
   char date[20];

   time (&current);
   timestruct = localtime (&current);

   strftime (date, sizeof (date), "%d %b %Y", timestruct);
   XtVaSetValues (wids->TF1, XmNvalue, date, NULL);
   XmProcessTraversal (wids->next, XmTRAVERSE_CURRENT);
}


/*
 * startCB Callback
 * Puts the date and time into the date and start time text fields
 * and forces focus to the Call text field.
 */
void startCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Cdata *wids = (Cdata *) cdata;
   time_t current;
   struct tm *timestruct = NULL;
   char date[20];
   char stime[20];

   time (&current);
   timestruct = localtime (&current);

   strftime (date, sizeof (date), "%d %b %Y", timestruct);
   strftime (stime, sizeof (stime), "%H%M %Z", timestruct);

   XtVaSetValues (wids->TF1, XmNvalue, date, NULL);

   XtVaSetValues (wids->TF2, XmNvalue, stime, NULL);

   XmProcessTraversal (wids->next, XmTRAVERSE_CURRENT);
}


/*
 * traversCB Callback
 * Forces traversal to the next widget.
 */
void traverseCB (Widget w, XtPointer cdata, XtPointer cbstruct)
{
   XmProcessTraversal (w, XmTRAVERSE_NEXT_TAB_GROUP);
}


/*
 * callsignCB Callback
 * modify/verify callback changes all chars to upper case
 * activate callback - Moves focus to next tab group, the Band Menu.
 * losing focus callback - shares call and starts search 
 */
void callsignCB (Widget w, XtPointer cdata, XtPointer cbstruct)
{
   int i, j;
   static int begin = 1;
   XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct *) cbstruct;
   char *hisCall = NULL;

   switch (cbs->reason)
   {
      case XmCR_MODIFYING_TEXT_VALUE:

         if (cbs->text->length == 0)
         {
            /* leave doit true to handle backspace */
            return;
         }

         /* if a single char and alphanumeric or a '/', do it and return */
         if (cbs->text->length == 1)
         {
            if (isalnum (cbs->text->ptr[0]) || cbs->text->ptr[0] == '/')
            {
               /* change to uppercase */
               cbs->text->ptr[0] = toupper (cbs->text->ptr[0]);

               /* change 0 to slash zero */
               if (cbs->text->ptr[0] == '\330')
               {
                  cbs->text->ptr[0] = '\330';
               }
               return;
            }
            else      /* not valid call character, so ignore it */
            {
               cbs->doit = False;
               return;
            }
         }

         /* 
          * If we get here, it must be more than one char,
          * so handle the cut/past string  Just in case there
          * is a slash zero, change it to zero.
          */
         for (i=0; i<cbs->text->length; i++)
         {
            if (cbs->text->ptr[i] == '\330')
            {
               cbs->text->ptr[i] = '0';
            }
         }

         /* now trim non alphanumerics off the end */
         for (i=cbs->text->length - 1; i >= 0; i--)
         {
            if (!isalnum (cbs->text->ptr[i]))
            {
               cbs->text->ptr[i] = '\0';
            }
            else
            {
               cbs->text->length = strlen (cbs->text->ptr);  /* fix length */
               break;   /*not a space, so done */
            }
         }

         /* count leading white spaces and non valid chars to trim */
         j = 0;
         for (i=0; i<cbs->text->length; i++)
         {
            if (!isalnum(cbs->text->ptr[i]))
            {
               j++;
            }
            else
            {
               break;      /* found first valid char */
            }
         }

         /* move valid characters to the start of the string */
         if (j > 0)        /* if any non valid chars were found */
         {
            for (i=0; j<cbs->text->length; i++,j++)
            {
               cbs->text->ptr[i] = cbs->text->ptr[j];
            }
            cbs->text->ptr[i] = '\0';
            cbs->text->length = strlen (cbs->text->ptr);   /* fix length */
         }

         /* change chars to upper case - if zero, change to slashed zero */
         for (i=0; i<cbs->text->length; i++)
         {
            cbs->text->ptr[i] = toupper (cbs->text->ptr[i]);
            if (cbs->text->ptr[i] == '0')
            {
               cbs->text->ptr[i] = '\330';
            }
         }

         /* copy the string to shared memory - note: zero will be zero? */
         strncpy (calls->outCall, cbs->text->ptr, SHMSIZE); /* call to shm */
//printf ("str **%s**\n", cbs->text->ptr);
         /* force a search */
if (cbs->text->length > 1)
{
   callSearch (calls->outCall);
}
         break;

      case XmCR_ACTIVATE:      /* force traversal will cause losing focus */
         XmProcessTraversal (w, XmTRAVERSE_NEXT_TAB_GROUP);
         break;

      case XmCR_LOSING_FOCUS:      /* pass call to shared memory and search */
         XtVaGetValues (w,         /* to see if we worked him before */
            XmNvalue, &hisCall,
            NULL);
         strncpy (calls->outCall, hisCall, SHMSIZE); /* put it in shm */

         /* and search for His call */
         if (strlen (hisCall) > 1)      /* no search if only one char */
         {
            if (mwText == NULL)         /* build editor if it hasn't been */
            {
               build_editor ();
            }
            callSearch (hisCall);
         }

         XmProcessTraversal (w, XmTRAVERSE_NEXT_TAB_GROUP);
         break;
   }
}


/*
 * search for call sign
 * built the results matrix (mw) if it
 * does not exist.
 */
void callSearch (char *pattern)
{
   int i, k, rowcnt;
   static Widget mw;
   String str;
   String val;

   if (searchSH == NULL)        /* built it if it hasn't been */
   {
      mw = build_searchSH ();
   }

   XbaeMatrixDisableRedisplay (mw);   
   rowcnt = XbaeMatrixNumRows (mw);       /* clear search widget */
   XbaeMatrixDeleteRows (mw, 0, rowcnt);

   if (mwText == NULL)               /* build editor if we need one */
   {
      build_editor ();
   }
   rowcnt = XbaeMatrixNumRows (mwText);   /* */

   /* searching from botton up so recent first */
   for (i = rowcnt - 1; i >= 0; i--) 
   {
      str = XbaeMatrixGetCell (mwText, i, 3);
      if (strstr (str, pattern) != NULL)
      {
         XbaeMatrixAddRows (mw, 0, NULL, NULL, NULL, 1);
         for (k = 0; k <= 10; k++)
         {
            val = XbaeMatrixGetCell (mwText, i, k);
            XbaeMatrixSetCell (mw, 0, k, val);
         }
      }
   }

   if (XbaeMatrixNumRows (mw) > 0)
   {
      XtManageChild (searchSH);
   }
   else
   {
      XtUnmanageChild (searchSH);
   }

   XbaeMatrixEnableRedisplay (mw, True);
}


/*
 * build search results SH
 */
Widget build_searchSH (void)
{
   Widget form3, closePB, sep2, mwSearch;

   searchSH = XtVaCreatePopupShell ("searchSH", topLevelShellWidgetClass, logSH,
      XmNmwmDecorations, MWM_DECOR_RESIZEH | MWM_DECOR_TITLE | MWM_DECOR_BORDER,
      XmNtitle, "TWLOG CALL SEARCH",
      NULL);

   form3 = XtVaCreateWidget ("form3", xmFormWidgetClass, searchSH,
      NULL);

   closePB = XtVaCreateManagedWidget ("Close", xmPushButtonWidgetClass,
      form3,
      XmNwidth, PBWIDTH,
      XmNleftAttachment, XmATTACH_POSITION,
      XmNleftPosition, 50,
      XmNleftOffset, -PBWIDTH / 2,
      XmNbottomAttachment, XmATTACH_FORM, XmNbottomOffset, MARGIN, NULL);

   sep2 = XtVaCreateManagedWidget ("sep2", xmSeparatorWidgetClass, form3,
      XmNtopAttachment, XmATTACH_NONE,
      XmNleftAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_WIDGET,
      XmNbottomWidget, closePB,
      XmNbottomOffset, MARGIN, XmNrightAttachment, XmATTACH_FORM, NULL);


   mwSearch =
      XtVaCreateManagedWidget ("mwSearch", xbaeMatrixWidgetClass, form3,
      XmNtopAttachment, XmATTACH_FORM, XmNleftAttachment, XmATTACH_FORM,
      XmNbottomAttachment, XmATTACH_WIDGET, XmNbottomWidget, sep2,
      XmNbottomOffset, MARGIN, XmNrightAttachment, XmATTACH_FORM,
      XmNcalcCursorPosition, True, XmNvisibleRows, 3, NULL);

   XtAddCallback (closePB, XmNactivateCallback, closeCB, (XtPointer) NULL);
   XtAddCallback (mwSearch, XmNenterCellCallback, enterCellCB,
      (XtPointer) NULL);
   XtManageChild (form3);
   return (mwSearch);
}


/*
 * enterCellCB - makes call search matrix non editable
 * setting map in the cbs to false is all it takes
 */
void enterCellCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XbaeMatrixEnterCellCallbackStruct *pt =
      (XbaeMatrixEnterCellCallbackStruct *) cbs;

   pt->map = False;
}


/*
 * closeCB - closes the call search shell
 */
void closeCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XtUnmanageChild (searchSH);
}


/*
 * endCB Callback
 * Puts the time into the End Time text field
 * and moves the focus to the Date Pushbutton.
 */
void endCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   Cdata *wids = (Cdata *) cdata;
   time_t current;
   struct tm *timestruct = NULL;
   char stime[20];

   time (&current);
   timestruct = localtime (&current);

   strftime (stime, sizeof (stime), "%H%M %Z", timestruct);
   XtVaSetValues (wids->TF1, XmNvalue, stime, NULL);
   XmProcessTraversal (wids->next, XmTRAVERSE_CURRENT);
}


/*
 * focusCB Callback
 * This callback is called for any loss or gain of focus in any
 * text field in the main window.  The I beam cursor will only be
 * visible if the textfield has focus.
 */
void focusCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   XmAnyCallbackStruct *cbstruct = (XmAnyCallbackStruct *) cbs;

   switch (cbstruct->reason)
   {
      case XmCR_FOCUS:
         XtVaSetValues (w, XmNcursorPositionVisible, TRUE, NULL);
         break;

      case XmCR_LOSING_FOCUS:
         XtVaSetValues (w, XmNcursorPositionVisible, FALSE, NULL);
         break;

      default:
         fprintf (stderr, "Invalid focusCB reason\n");
         break;
   }
}


/*
 * fileCB Callback
 * All callbacks on the file pulldown come here.  The button number
 * is checked to see which option was selected.
 */
void fileCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   int rtn;
   long btn = (long) cdata;
   char tail[MAXNAME];
   char *newname;
   char *lprstr;
   time_t current;
   struct tm *timestruct = NULL;
   
   switch (btn)
   {
      case PRINT:
         if ((lprstr = malloc (sizeof (PRINT_CMD)
                  + sizeof (logpath) + MAXNAME)) == NULL)
         {
            perror ("fileCB - print malloc failed");
            exit (1);
         }
         strcpy (lprstr, PRINT_CMD);
         strcat (lprstr, logpath);
         rtn = system (lprstr);
         free (lprstr);
         break;

      case NEW:
         time (&current);
         timestruct = localtime (&current);
         strftime (tail, MAXNAME, "/log.%Y.%j.%H%M%S", timestruct);

         if ((newname = malloc (strlen (dirpath) + MAXNAME)) == NULL)
         {
            perror ("newname malloc failed:");
            exit (1);
         }
         strcpy (newname, dirpath);
         strcat (newname, tail);

         if (rename (logpath, newname) == 0)
            creat (logpath, 0644);
         else
         {
            perror ("Rename failed");
            exit (1);
         }
         free (newname);
         sleep (1);             /* paranoia - prevents the same name */
         break;
         
      case CONVERT:
            build_convDiag();
            break;

      case QRT:
         shmdt (calls);
         shmctl (shmid1, IPC_RMID, NULL);
         exit (0);
   }
}


/*
 * editCB Callback
 * All callbacks on the file pulldown come here.  The button number
 * is checked to see which option was selected.
 */
void editCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   long btn = (long) cdata;
   int rtn;
   Widget *col_2_wids;

   switch (btn)
   {
      case APPEND:
         XtVaGetValues (XtParent (w), XmNuserData, &col_2_wids, NULL);

         if (appendlog (logpath, col_2_wids) == 0)
         {
            break;  /* don't clear form on error */
         }
         clearform (col_2_wids);
         break;

      case CLEAR:
         XtVaGetValues (XtParent (w), XmNuserData, &col_2_wids, NULL);
         clearform (col_2_wids);
         break;

      case EDIT:
         popupEdit (logpath);
         break;
   }
}


/*
 * appendlog Function
 * Collects the data from the righthand column and writes it
 * to the end of logfile.
 */

int appendlog (char logpath[], Widget * col_2_wids)
{
   int i;
   FILE *fp;
   Widget btnwid;
   XmString label_xs;
   char *data;
   char abox[9][MAXBOX + 1];
   char anote[MAXNOTE + 1];

   for (i = 0; i < 9; i++)
   {
      if (i < 3 || i > 5)
      {
         XtVaGetValues (col_2_wids[i],  /* get text from a textfield */
            XmNvalue, &data,
            NULL);
         strncpy (abox[i], data, MAXBOX);
      }
      else   /* or get XmString from a menu */
      {
         XtVaGetValues (col_2_wids[i],  /* get the menu button wid, then */
            XmNmenuHistory, &btnwid,
            NULL);
         XtVaGetValues (btnwid,         /* get label for selected button */
            XmNlabelString, &label_xs,
            NULL);
         data = XmStringUnparse (label_xs, NULL, XmCHARSET_TEXT, XmCHARSET_TEXT,
                                 NULL, 0, XmOUTPUT_ALL);
         strncpy (abox[i], data, MAXBOX);
      }
   }
   XtVaGetValues (col_2_wids[i],        /* get the Note field */
      XmNvalue, &data, NULL);
   strncpy (anote, data, MAXNOTE);

   /*
    * call send_eqsl if toggle button is set
    */
   if (XmToggleButtonGetState (col_2_wids[10]) == 1)
   {
      if (send_eqsl(abox) == 0)
      {
         return 0;
      }
   }

   if ((fp = fopen (logpath, "a")) == NULL)     /* open logfile to append */
   {
      perror ("appendlog - open failed");
      exit (1);
   }

   fprintf (fp, "%-15s%-10s%-15s%-15s%-10s%-10s%-10s%-10s%-10s%-1s",
      abox[0], abox[1], abox[2], abox[3], abox[4],
      abox[5], abox[6], abox[7], abox[8], anote);
   fprintf (fp, "\n");

   fclose (fp);   /* and close logfile */
   return 1;
}


/*
 * clearform Function
 * Clears all text field widgets
 */
void clearform (Widget * col_2_wids)
{
   int i;

   for (i = 0; i <= 9; i++)
   {
      if (i < 3 || i > 5)
         XtVaSetValues (col_2_wids[i], XmNvalue, "", NULL);
   }

   /* clear the callsign shared memory if set */
   if (calls != NULL)
   {
      strcpy (calls->outCall, "");
   }
}


/*
 * helpCB Callback
 * All callbacks on the help pulldown come here.  The button number
 * is checked to see which option was selected.
 */
void helpCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   long btn = (long) cdata;
   switch (btn)
   {
      case ABOUT:
         aboutDiag ();
         break;

      default:
         popupHelp (btn);
         break;
   }
}


/*
 * initCB Callback - focus CB on form1
 * Initialize dirpath and logpath.  If they don't exist, popup a 
 * dialog to see if we really want to create them. 
 */
void initCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   struct stat buf;
   long eflag = 0;
   Widget initDiag;
   XmString xs;
   char *mess =
           "Twlog needs to create .twlogDir/logfile in your home directory.\n"
           "Click OK to create it or Cancel to exit.\n";

   /* remove this callback so it is only called once at start up */
   XtRemoveCallback (w, XmNfocusCallback, initCB, NULL);

   if (stat (getenv ("HOME"), &buf) == -1)      /* Do you have a $HOME? */
   {
      fprintf (stderr, "initCB: Can't find your \"$HOME\"\n");  /* poor baby */
      exit (1);
   }

   strcpy (dirpath, getenv ("HOME"));   /* $HOME to dir path */
   strcat (dirpath, LOGDIR);            /* then add LOGDIR */
   if ((stat (dirpath, &buf)) == -1)
   {  /* LOGDIR does not exist */
      fprintf (stderr, "initCB: Can't find log directory %s\n", dirpath);
      eflag++;
   }
   else
   {
      /* it exists, but is it a dir? */
      if (!(S_ISDIR(buf.st_mode)))
      {
         fprintf (stderr, "initCB: twlogDir exists, but isn't a directory\n");
         exit (1);
      }
   }

   strcpy (logpath, dirpath);           /* dirpath to logpath */
   strcat (logpath, LOGFILE);           /* add LOGFILE name   */
   if (stat (logpath, &buf) == -1)      /* and check that it exists */
   {
      fprintf (stderr, "initCB: Can't find %s\n", logpath);
      eflag++;
   }

   if (eflag > 0)
   {
      /* create and popup the dialog */
      /* use below - via Jacob KD7YKO xs = XmStringCreateLocalized (mess); */
      xs = XmStringCreateLtoR (mess, XmSTRING_DEFAULT_CHARSET);
      initDiag = XmCreateQuestionDialog (logSH, "initDiag", NULL, 0);

      XtVaSetValues (XtParent (initDiag),
         XmNtitle, "CREATE LOGFILE",
         #if MAKE_ICON && HAVE_X11_XPM_H && HAVE_LIBXPM
         XmNiconPixmap, pixmap,
         #endif
         NULL);

      XtVaSetValues (initDiag,
         XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
         XmNmessageString, xs,
         NULL);
      XmStringFree (xs);
      XtUnmanageChild (XmMessageBoxGetChild (initDiag, XmDIALOG_HELP_BUTTON));

      /* add its callbacks */
      XtAddCallback (initDiag, XmNokCallback, initDiagCB, (XtPointer)eflag);
      XtAddCallback (initDiag, XmNcancelCallback, initDiagCB, (XtPointer)eflag);
      XtManageChild (initDiag);
   }
}


/*
 * initDiagCB - OK and Cancel callback for initDiag
 * cdata is eflag from initDiag
 * eflag==1: logfile is missing 
 * eflag==2: both logpath and dirpath are missing 
 */
void initDiagCB (Widget w, XtPointer cdata, XtPointer cbs)
{
   long eflag = (long) cdata;
   int modeDir = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
   int modeLog = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;

   XmAnyCallbackStruct *ptr = (XmAnyCallbackStruct *) cbs; 

   switch (ptr->reason)
   {
      case XmCR_CANCEL:
         exit (1);      /* Cancel - quit twlog */
         break;

      case XmCR_OK:
         switch (eflag)
         {
            case 2:
               mkdir (dirpath, modeDir);
               /* No break - we need to fall through */
            case 1:
               creat (logpath, modeLog);
               break;
            default:
               fprintf (stderr, "initDiagCB: invalid eflag\n");
               exit (1);
         }
         break;   
      default:
         fprintf (stderr, "initDiagCB - invalid CB reason\n");
         exit (1);
   }
}


/*
 * popupHandler - Event handler to popup menu
 */
void popupHandler (Widget w, XtPointer cdata, XEvent * event, Boolean * cont)
{
   Widget menuWid = (Widget) cdata;

   if (event->type == ButtonPress && event->xbutton.button == Button3)
   {
      XmMenuPosition (menuWid, &(event->xbutton));
      XtManageChild (menuWid);
   }
}


/*
 * timeout process
 */
void timeoutCB (XtPointer cdata, XtIntervalId *id)
{
   XtAppContext ac = (XtAppContext) cdata;
   char *callsign;
   char *notes2;
   char notes[128] = "\0";
   int val;

   static struct sembuf sops[] = {
                                   {1, 1, SEM_UNDO},  /* inc the counter */
                                   {1, -1, SEM_UNDO}  /* dec the counter */
                                 };

   /* get the semaphore id */
   if ((semid = semget (SEM_KEY, 2, 0)) == -1)
   {
      /* no server online - maybe later */
      XtAppAddTimeOut (ac, timeout, timeoutCB, ac);
      return;
   }

   /* check sem 0 to see if server has anything for us */
   if (semctl (semid, 0, GETVAL, 0) == 0)
   {
      /* no data from server */
      XtAppAddTimeOut (ac, timeout, timeoutCB, ac);
      return;
   }
   else
   {
      /* got data, so fetch the shared memory id */
      if ((shmid2 = shmget (XML_KEY, 0, 0)) == -1)
      {
         perror ("timeoutCB: shmget failed");
         exit (1);
      }

      /* attach the shared memory */
      if ((XMLdata = (struct allXML *)shmat (shmid2, NULL, 0)) ==
                     (struct allXML *) NULL)
      {
         /* can't attach the shared memory - bad news */
         perror ("timeoutCB shmat failed");
         exit (1);
      }

      /* get current callsign and notes from the GUI */
      XtVaGetValues (timerData.callsignTF,
         XmNvalue, &callsign,
         NULL);

      XtVaGetValues (timerData.notesTF,
         XmNvalue, &notes2,
         NULL);

      /* generate new notes - used here and below */
      if (strlen (XMLdata->us_state) == 2)
      {
         sprintf (notes, "%s %s   %s",
                 XMLdata->qth, XMLdata->us_state, XMLdata->nick); 
      }
      else if (strlen (XMLdata->district) >= 1)
      {
         sprintf (notes, "%s %s %s   %s",
            XMLdata->qth, XMLdata->district, XMLdata->country, XMLdata->nick);
      }
      else
      {
         sprintf (notes, "%s %s   %s",
            XMLdata->qth, XMLdata->country, XMLdata->nick);
      }

      /* compare new and gui callsign and notes - return if equal */ 
      if ((strcmp (callsign, XMLdata->callsign) == 0) &&
         (strcmp (notes2, notes) == 0))
      {
         /* not new data, so detatch memory and restart timeout */
         shmdt (XMLdata);
         XtAppAddTimeOut (ac, timeout, timeoutCB, ac);
         return;
      }
   }

   /* got new data, so increment the client sem */
   if (semop (semid, &sops[0], 1) == -1)
   {
      perror ("timeoutCB - increment semaphore failed");
      exit (1);
   }

//fprintf (stderr, "sem1 cnt + %d\n", semctl (semid, 1, GETVAL, 0));

   /* all ready now. copy the data to the GUI */
   XtVaSetValues (timerData.callsignTF,
      XmNvalue, XMLdata->callsign,
      NULL);

   XtVaSetValues (timerData.notesTF,
      XmNvalue, notes,
      NULL);

   /* decrement the counter */
   if (semop (semid, &sops[1], 1) == -1)
   {
      perror ("timeoutCB - decrement semaphore failed");
      exit (1);
   }

//fprintf (stderr, "sem1 cnt - %d\n", semctl (semid, 1, GETVAL, 0));

   /* now force a search for His call */
   if (strlen (XMLdata->callsign) > 1)  /* no search if only one char */
   {
      if (mwText == NULL)               /* build editor if we need one */
      {
         build_editor ();
      }
      callSearch (XMLdata->callsign);
   }

   /* detatch the shared memory */
   shmdt (XMLdata);
   XtAppAddTimeOut (ac, timeout, timeoutCB, ac);
   return;
}


/*
 * send_eqsl - given qsl data, generate an eQSL
 */
#define ESTRSIZE 3

int send_eqsl(char abox[][MAXBOX+1])
{
   FILE *fp;
   char *date, *mon, *year, *fmtp;
   int  i, imonth, idate;
   char buff[512];
   char request[400];

   char *months[] = {"BAD", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                            "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

   char *emsg[ESTRSIZE] = 
      {
         "Information:",                                   /*0*/
         "Error: ",                                        /*1*/
         "Warning: "                                       /*2*/
      };

   /* two format strings.  first if BAND data, second if FREQ */
   const char *fmt[] = 
      {
         "wget -q -O - -t 2 \""
         "http://www.eqsl.cc/qslcard/importADIF.cfm?ADIFdata=upload"
         "<adIF_ver:3>2.2<EQSL_USER:%d>%s<EQSL_PSWD:%d>%s"
         "<PROGRAMID:5>twlog<EOH><BAND:%d>%s<CALL:%d>%s"
         "<MODE:%d>%s<QSO_DATE:8>%s%02d%02d"
         "<TIME_ON:4>%s"
         "<RST_SENT:%d>%s<EOR>\"",

         "wget -q -O - -t 2 \""
         "http://www.eqsl.cc/qslcard/importADIF.cfm?ADIFdata=upload"
         "<adIF_ver:5>2.1.9<EQSL_USER:%d>%s<EQSL_PSWD:%d>%s"
         "<PROGRAMID:5>twlog<EOH><FREQ:%d>%s<CALL:%d>%s"
         "<MODE:%d>%s<QSO_DATE:8>%s%02d%02d"
         "<TIME_ON:4>%s"
         "<RST_SENT:%d>%s<EOR>\""
      };

   /* make sure the call looks valid */
   if (strlen (abox[2]) < 3)
   {
      errorDiag (logSH, "Please enter His Call.", !CANCEL_BTN);
      return 0;
   }

   /* make sure there is a His RST entry */
   if (strlen (abox[6]) == 0)
   {
      errorDiag (logSH, "Please enter His RST.", !CANCEL_BTN);
      return 0;
   }
      
   /* trim date */
   abox[0][2] = '\0';
   date = &abox[0][0];
   idate = atoi (date);

   /* trim month and convert to idate */
   abox[0][6] = '\0';
   mon = &abox[0][3];
   /* find month number */
   for (imonth=1; imonth<=12; imonth++)
   {
      if (strcmp (mon, months[imonth]) == 0)
      {
         break;
      }
   }

   /* and the year */
   year = &abox[0][7];

   /* trim time to 4 digits to remove time zone */
   abox[1][4] = '\0';

   /* make sure Band is upercase M */
   for (i=0; i<strlen(abox[3]); i++)
   {
      toupper (abox[3][i]);
   }
   /* then check abox[3] to see if we need BAND or FREQ format */
   i = 0;
   if (strstr (abox[3], "M") == NULL)
   {
      i = 1;
   }
   sprintf (request, fmt[i],
      strlen (user), user,
      strlen (pword), pword,
      strlen (abox[3]), abox[3],
      strlen (abox[2]), abox[2],
      strlen (abox[4]), abox[4],
      year, imonth, idate,
      abox[1],
      strlen (abox[6]), abox[6]);

   /* repair the damages */
   abox[0][2] = ' ';
   abox[0][6] = ' ';
   abox[1][4] = ' ';

//printf ("%s\n", request);
   /* all set, so send the eqsl info */
   fp = popen (request, "r");
   if (fp < 0)
   {
      fprintf (stderr, "send_eqsl: popen failed");
      errorDiag (logSH, "send_eqsl: popen failed", !CANCEL_BTN);
      return 0;
   }

   /* get the reply from eqsl */
   while ((fgets (buff, sizeof (buff), fp)) > 0)
   {
//printf ("%s", buff);
      for (i=1; i<ESTRSIZE; i++)
      {
         if (strstr (buff, emsg[i]) != NULL)
         {
            buff[strlen(buff) - 2] = '\n';  /* clean up the end - 2 works */
            errorDiag (logSH, buff, !CANCEL_BTN);
            return 0;
         }
      }
   }
   pclose (fp);
   return 1;
}
 
